@@ -49,6 +49,20 @@
 #define PATH_MAX (MAX_PATH)
 #endif
 
+/* Default settings */
+#if defined(WIN32) || defined(DJGPP)
+#  define DEFAULT_DEVICE "com2:"
+#else
+#  define DEFAULT_DEVICE "/dev/ttyUSB0"
+#endif
+#define DEFAULT_MODEL ""
+#define DEFAULT_CONNECTION "at"
+#define DEFAULT_SYNCHRONIZE_TIME FALSE
+#define DEFAULT_DEBUG_FILE ""
+#define DEFAULT_DEBUG_LEVEL ""
+#define DEFAULT_LOCK_DEVICE FALSE
+#define DEFAULT_START_INFO FALSE
+
 /**
  * Returns current debuging descriptor. It honors use_global
  * flag.
@@ -140,6 +154,14 @@ static const GSM_ConnectionInfo GSM_Connections[] = {
 	{"blues60", GCT_BLUES60, FALSE},
 	{"bluerfs60", GCT_BLUES60, FALSE},
 
+	/* proxy */
+	{"proxyobex", GCT_PROXYOBEX, FALSE},
+	{"proxyphonet", GCT_PROXYPHONET, FALSE},
+	{"proxyat", GCT_PROXYAT, FALSE},
+	{"proxygnapbus", GCT_PROXYGNAPBUS, FALSE},
+	{"proxyfbus", GCT_PROXYFBUS2, FALSE},
+	{"proxys60", GCT_PROXYS60, FALSE},
+
 	/* old "serial" irda */
 	{"infrared", GCT_FBUS2IRDA, FALSE},
 	{"fbusirda", GCT_FBUS2IRDA, FALSE},
@@ -306,6 +328,26 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 #endif
 #ifdef GSM_ENABLE_BLUEOBEX
 	GSM_RegisterConnection(s, GCT_BLUEOBEX,   &BlueToothDevice,&OBEXProtocol);
+#endif
+#if !defined(WIN32) && defined(HAVE_PTHREAD)
+#ifdef GSM_ENABLE_S60
+	GSM_RegisterConnection(s, GCT_PROXYS60,	  &ProxyDevice,&S60Protocol);
+#endif
+#ifdef GSM_ENABLE_BLUEGNAPBUS
+	GSM_RegisterConnection(s, GCT_PROXYGNAPBUS,&ProxyDevice,&GNAPBUSProtocol);
+#endif
+#ifdef GSM_ENABLE_FBUS2
+	GSM_RegisterConnection(s, GCT_PROXYFBUS2,  &ProxyDevice,&FBUS2Protocol);
+#endif
+#ifdef GSM_ENABLE_DKU2PHONET
+	GSM_RegisterConnection(s, GCT_PROXYPHONET, &ProxyDevice,&PHONETProtocol);
+#endif
+#ifdef GSM_ENABLE_ATGEN
+	GSM_RegisterConnection(s, GCT_PROXYAT, 	  &ProxyDevice,&ATProtocol);
+#endif
+#ifdef GSM_ENABLE_OBEXGEN
+	GSM_RegisterConnection(s, GCT_PROXYOBEX,   &ProxyDevice,&OBEXProtocol);
+#endif
 #endif
 	if (s->Device.Functions == NULL || s->Protocol.Functions == NULL) {
 		smprintf(s, "Connection %s is know but was disabled on compile time\n", connection);
@@ -347,7 +389,7 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 		model = GetModelData(s, NULL, s->Phone.Data.Model, NULL);
 #ifdef GSM_ENABLE_ATGEN
 		/* With ATgen and auto model we can work with unknown models too */
-		if (s->ConnectionType==GCT_AT || s->ConnectionType==GCT_BLUEAT || s->ConnectionType==GCT_IRDAAT || s->ConnectionType==GCT_DKU2AT) {
+		if (s->ConnectionType==GCT_AT || s->ConnectionType==GCT_PROXYAT || s->ConnectionType==GCT_IRDAAT || s->ConnectionType==GCT_DKU2AT) {
 #ifdef GSM_ENABLE_ALCATEL
 			/* If phone provides Alcatel specific functions, enable them */
 			if (model->model[0] != 0 && GSM_IsPhoneFeatureAvailable(model, F_ALCATEL)) {
@@ -371,7 +413,7 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 #endif
 		/* With OBEXgen and auto model we can work with unknown models too */
 #ifdef GSM_ENABLE_OBEXGEN
-		if (s->ConnectionType==GCT_BLUEOBEX || s->ConnectionType==GCT_IRDAOBEX) {
+		if (s->ConnectionType==GCT_BLUEOBEX || s->ConnectionType==GCT_PROXYOBEX || s->ConnectionType==GCT_IRDAOBEX) {
 			smprintf(s,"[Module           - \"%s\"]\n",OBEXGENPhone.models);
 			s->Phone.Functions = &OBEXGENPhone;
 			return ERR_NONE;
@@ -388,14 +430,14 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 
 		/* With GNAPgen and auto model we can work with unknown models too */
 #ifdef GSM_ENABLE_GNAPGEN
-		if (s->ConnectionType == GCT_BLUEGNAPBUS || s->ConnectionType == GCT_IRDAGNAPBUS) {
+		if (s->ConnectionType == GCT_BLUEGNAPBUS || s->ConnectionType == GCT_PROXYGNAPBUS || s->ConnectionType == GCT_IRDAGNAPBUS) {
 			smprintf(s,"[Module           - \"%s\"]\n",GNAPGENPhone.models);
 			s->Phone.Functions = &GNAPGENPhone;
 			return ERR_NONE;
 		}
 #endif
 #ifdef GSM_ENABLE_S60
-		if (s->ConnectionType == GCT_BLUES60) {
+		if (s->ConnectionType == GCT_BLUES60 || s->ConnectionType == GCT_PROXYS60) {
 			smprintf(s,"[Module           - \"%s\"]\n",S60Phone.models);
 			s->Phone.Functions = &S60Phone;
 			return ERR_NONE;
@@ -415,6 +457,8 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 				s->ConnectionType ==  GCT_PHONETBLUE ||
 				s->ConnectionType ==  GCT_IRDAPHONET ||
 				s->ConnectionType ==  GCT_BLUEFBUS2 ||
+				s->ConnectionType ==  GCT_PROXYFBUS2 ||
+				s->ConnectionType ==  GCT_PROXYPHONET ||
 				s->ConnectionType ==  GCT_BLUEPHONET) {
 			/* Try to detect phone type */
 			if (strcmp(model->model, "unknown") == 0 && model->features[0] == 0) {
@@ -459,7 +503,7 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 	s->Phone.Functions = NULL;
 #ifdef GSM_ENABLE_ATGEN
 	/* AT module can have the same models ID to "normal" Nokia modules */
-	if (s->ConnectionType==GCT_AT || s->ConnectionType==GCT_BLUEAT || s->ConnectionType==GCT_IRDAAT || s->ConnectionType==GCT_DKU2AT) {
+	if (s->ConnectionType==GCT_AT || s->ConnectionType==GCT_PROXYAT || s->ConnectionType==GCT_BLUEAT || s->ConnectionType==GCT_IRDAAT || s->ConnectionType==GCT_DKU2AT) {
 		GSM_RegisterModule(s,&ATGENPhone);
 		if (s->Phone.Functions != NULL) return ERR_NONE;
 	}
@@ -595,6 +639,7 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 #ifdef GSM_ENABLE_ATGEN
 			case GCT_AT:
 			case GCT_BLUEAT:
+			case GCT_PROXYAT:
 			case GCT_IRDAAT:
 			case GCT_DKU2AT:
 				s->Phone.Functions = &ATGENPhone;
@@ -602,18 +647,21 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 #endif
 #ifdef GSM_ENABLE_OBEXGEN
 			case GCT_IRDAOBEX:
+			case GCT_PROXYOBEX:
 			case GCT_BLUEOBEX:
 				s->Phone.Functions = &OBEXGENPhone;
 				break;
 #endif
 #ifdef GSM_ENABLE_GNAPGEN
 			case GCT_BLUEGNAPBUS:
+			case GCT_PROXYGNAPBUS:
 			case GCT_IRDAGNAPBUS:
 				s->Phone.Functions = &GNAPGENPhone;
 				break;
 #endif
 #ifdef GSM_ENABLE_S60
 			case GCT_BLUES60:
+			case GCT_PROXYS60:
 				s->Phone.Functions = &S60Phone;
 				break;
 #endif
@@ -630,7 +678,9 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 			case GCT_PHONETBLUE:
 			case GCT_IRDAPHONET:
 			case GCT_BLUEFBUS2:
+			case GCT_PROXYFBUS2:
 			case GCT_BLUEPHONET:
+			case GCT_PROXYPHONET:
 				s->Phone.Functions = &NAUTOPhone;
 				break;
 #endif
@@ -712,12 +762,7 @@ GSM_Error GSM_InitConnection_Log(GSM_StateMachine *s, int ReplyNum, GSM_Log_Func
 			}
 		}
 
-		smprintf_level(s, D_ERROR, "[Gammu            - %s built %s %s using %s]\n",
-				GAMMU_VERSION,
-				__TIME__,
-				__DATE__,
-				GetCompiler()
-				);
+		smprintf_level(s, D_ERROR, "[Gammu            - %s]\n", GAMMU_VERSION);
 		StripSpaces(s->CurrentConfig->Connection);
 		StripSpaces(s->CurrentConfig->Model);
 		StripSpaces(s->CurrentConfig->Device);
@@ -751,8 +796,11 @@ autodetect:
 				s->ConnectionType != GCT_NONE &&
 				s->ConnectionType != GCT_IRDAOBEX &&
 				s->ConnectionType != GCT_BLUEOBEX &&
+				s->ConnectionType != GCT_PROXYOBEX &&
 				s->ConnectionType != GCT_BLUEGNAPBUS &&
+				s->ConnectionType != GCT_PROXYGNAPBUS &&
 				s->ConnectionType != GCT_IRDAGNAPBUS &&
+				s->ConnectionType != GCT_PROXYS60 &&
 				s->ConnectionType != GCT_BLUES60) {
 			error = GSM_TryGetModel(s);
 			/* Fall back to other configuraitons if the device is not existing (or similar error) */
@@ -851,6 +899,12 @@ autodetect:
 			return error;
 		}
 
+		error=s->Phone.Functions->PostConnect(s);
+		if (error != ERR_NONE && error != ERR_NOTSUPPORTED) {
+			GSM_LogError(s, "Init:Phone->PostConnect" , error);
+			return error;
+		}
+
 		smprintf(s,"[Connected]\n");
 		return ERR_NONE;
 	}
@@ -873,8 +927,8 @@ int GSM_ReadDevice (GSM_StateMachine *s, gboolean waitforreply)
 	}
 
 	GSM_GetCurrentDateTime (&Date);
-	i=Date.Second;
-	while (i==Date.Second && !s->Abort) {
+	i = Date.Second;
+	while (i == Date.Second && !s->Abort) {
 		res = s->Device.Functions->ReadDevice(s, buff, sizeof(buff));
 
 		if (!waitforreply) {
@@ -887,7 +941,7 @@ int GSM_ReadDevice (GSM_StateMachine *s, gboolean waitforreply)
 		GSM_GetCurrentDateTime(&Date);
 	}
 	for (count = 0; count < res; count++) {
-		s->Protocol.Functions->StateMachine(s,buff[count]);
+		s->Protocol.Functions->StateMachine(s, buff[count]);
 	}
 	return res;
 }
@@ -941,7 +995,7 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 			sentmsg.Length 	= length;
 			sentmsg.Type	= type;
 			sentmsg.Buffer 	= (unsigned char *)malloc(length);
-			memcpy(sentmsg.Buffer,buffer,length);
+			memcpy(sentmsg.Buffer, buffer, length);
 			Phone->SentMsg  = &sentmsg;
 		}
 
@@ -949,9 +1003,6 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 		if (GSM_ReadDevice(s, TRUE) > 0) {
 			i = 0;
 		} else {
-			if (s->Abort) {
-				return ERR_ABORTED;
-			}
 			usleep(10000);
 		}
 
@@ -961,12 +1012,16 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 			Phone->SentMsg = NULL;
 		}
 
+		if (s->Abort) {
+			return ERR_ABORTED;
+		}
+
 		/* Request completed */
-		if (Phone->RequestID==ID_None) {
+		if (Phone->RequestID == ID_None) {
 			return Phone->DispatchError;
 		}
 		i++;
-	} while (i<timeout);
+	} while (i < timeout);
 
 	return ERR_TIMEOUT;
 }
@@ -982,19 +1037,23 @@ GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
 	if (s->CurrentConfig->StartInfo) {
 		if (Phone->StartInfoCounter > 0) {
 			Phone->StartInfoCounter--;
-			if (Phone->StartInfoCounter == 0) s->Phone.Functions->ShowStartInfo(s,FALSE);
+			if (Phone->StartInfoCounter == 0) {
+				s->Phone.Functions->ShowStartInfo(s,FALSE);
+			}
 		}
 	}
 
 	Phone->RequestID	= request;
 	Phone->DispatchError	= ERR_TIMEOUT;
 
-	for (reply=0;reply<s->ReplyNum;reply++) {
-		if (reply!=0) {
+	for (reply = 0; reply < s->ReplyNum; reply++) {
+		if (reply != 0) {
 			smprintf_level(s, D_ERROR, "[Retrying %i type 0x%02X]\n", reply, type);
 		}
 		error = s->Protocol.Functions->WriteMessage(s, buffer, length, type);
-		if (error!=ERR_NONE) return error;
+		if (error != ERR_NONE) {
+			return error;
+		}
 
 		/* Special case when no reply is expected */
 		if (request == ID_None) {
@@ -1002,9 +1061,16 @@ GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
 		}
 
 		error = GSM_WaitForOnce(s, buffer, length, type, timeout);
-		if (error != ERR_TIMEOUT) return error;
+		if (error != ERR_TIMEOUT) {
+			return error;
+		}
         }
 
+	if (request != ID_Reset && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_RESET_AFTER_TIMEOUT)) {
+		smprintf_level(s, D_ERROR, "Performing device reset after timeout!\n");
+		GSM_Reset(s, FALSE);
+	}
+
 	return ERR_TIMEOUT;
 }
 
@@ -1016,8 +1082,8 @@ static GSM_Error CheckReplyFunctions(GSM_StateMachine *s, GSM_Reply_Function *Re
 	gboolean				available = FALSE;
 	int				i	  = 0;
 
-	while (Reply[i].requestID!=ID_None) {
-		execute=FALSE;
+	while (Reply[i].requestID != ID_None) {
+		execute = FALSE;
 		/* Long ID frames like S60 */
 		if (Reply[i].msgtype[0] == 0 && Reply[i].subtypechar == 0) {
 			if (Reply[i].subtype == msg->Type) {
@@ -1031,7 +1097,9 @@ static GSM_Error CheckReplyFunctions(GSM_StateMachine *s, GSM_Reply_Function *Re
 						if (msg->Buffer[Reply[i].subtypechar]==Reply[i].subtype)
 							execute=TRUE;
 					}
-				} else execute=TRUE;
+				} else {
+					execute=TRUE;
+				}
 			}
 		} else {
 			if (strlen(Reply[i].msgtype) < msg->Length) {
@@ -1042,13 +1110,13 @@ static GSM_Error CheckReplyFunctions(GSM_StateMachine *s, GSM_Reply_Function *Re
 		}
 
 		if (execute) {
-			*reply=i;
+			*reply = i;
 			if (Reply[i].requestID == ID_IncomingFrame ||
 			    Reply[i].requestID == Data->RequestID ||
 			    Data->RequestID    == ID_EachFrame) {
 				return ERR_NONE;
 			}
-			available=TRUE;
+			available = TRUE;
 		}
 		i++;
 	}
@@ -1069,26 +1137,26 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 	GSM_Reply_Function	*Reply;
 	int			reply;
 
-	GSM_DumpMessageLevel2Recv(s, msg->Buffer, msg->Length, msg->Type);
-	GSM_DumpMessageLevel3Recv(s, msg->Buffer, msg->Length, msg->Type);
+	GSM_DumpMessageTextRecv(s, msg->Buffer, msg->Length, msg->Type);
+	GSM_DumpMessageBinaryRecv(s, msg->Buffer, msg->Length, msg->Type);
 
 	Reply = s->User.UserReplyFunctions;
 	if (Reply != NULL) {
-		error = CheckReplyFunctions(s,Reply,&reply);
+		error = CheckReplyFunctions(s, Reply, &reply);
 	}
 
 	if (error == ERR_UNKNOWNFRAME) {
 		Reply = s->Phone.Functions->ReplyFunctions;
-		error = CheckReplyFunctions(s,Reply,&reply);
+		error = CheckReplyFunctions(s, Reply, &reply);
 	}
 
-	if (error==ERR_NONE) {
-		error=Reply[reply].Function(msg, s);
-		if (Reply[reply].requestID==Phone->RequestID) {
+	if (error == ERR_NONE) {
+		error = Reply[reply].Function(msg, s);
+		if (Reply[reply].requestID == Phone->RequestID) {
 			if (error == ERR_NEEDANOTHERANSWER) {
 				error = ERR_NONE;
 			} else {
-				Phone->RequestID=ID_None;
+				Phone->RequestID = ID_None;
 			}
 		}
 	}
@@ -1265,7 +1333,7 @@ void GSM_ExpandUserPath(char **string)
 	if (home == NULL) return;
 
 	/* Allocate memory */
-	tmp = (char *)malloc(strlen(home) + strlen(*string));
+	tmp = (char *)malloc(strlen(home) + strlen(*string) + 2);
 	if (tmp == NULL) return;
 
 	/* Create final path */
@@ -1284,24 +1352,9 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 	gboolean	found = FALSE;
 	char		*Temp = NULL;
 
-#if defined(WIN32) || defined(DJGPP)
-        static const char *DefaultPort		= "com2:";
-#else
-        static const char *DefaultPort		= "/dev/ttyUSB0";
-#endif
-        static const char *DefaultModel		= "";
-        static const char *DefaultConnection		= "at";
-	static gboolean DefaultSynchronizeTime	= FALSE;
-	static const char *DefaultDebugFile		= "";
-	static const char *DefaultDebugLevel		= "";
-	static gboolean DefaultLockDevice		= FALSE;
-	static gboolean DefaultStartInfo		= FALSE;
-
-	/* By default all debug output will go to one filedescriptor */
-	static const gboolean DefaultUseGlobalDebugFile 	= TRUE;
 	GSM_Error error = ERR_UNKNOWN;
 
-	cfg->UseGlobalDebugFile	 = DefaultUseGlobalDebugFile;
+	cfg->UseGlobalDebugFile	 = TRUE;
 
 	/* If we don't have valid config, bail out */
 	if (cfg_info == NULL) {
@@ -1334,7 +1387,7 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 	if (!cfg->Device) {
 		cfg->Device 	 = INI_GetValue(cfg_info, section, "port", 		FALSE);
 		if (!cfg->Device) {
-			cfg->Device		 	 = strdup(DefaultPort);
+			cfg->Device		 	 = strdup(DEFAULT_DEVICE);
 		} else {
 			cfg->Device			 = strdup(cfg->Device);
 		}
@@ -1346,31 +1399,31 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 	free(cfg->Connection);
 	cfg->Connection  = INI_GetValue(cfg_info, section, "connection", 	FALSE);
 	if (cfg->Connection == NULL) {
-		cfg->Connection	 		 = strdup(DefaultConnection);
+		cfg->Connection	 		 = strdup(DEFAULT_CONNECTION);
 	} else {
 		cfg->Connection			 = strdup(cfg->Connection);
 	}
 
 	/* Set time sync */
-	cfg->SyncTime = INI_GetBool(cfg_info, section, "synchronizetime", DefaultSynchronizeTime);
+	cfg->SyncTime = INI_GetBool(cfg_info, section, "synchronizetime", DEFAULT_SYNCHRONIZE_TIME);
 
 	/* Set debug file */
 	free(cfg->DebugFile);
 	cfg->DebugFile   = INI_GetValue(cfg_info, section, "logfile", 		FALSE);
 	if (cfg->DebugFile == NULL) {
-		cfg->DebugFile		 	 = strdup(DefaultDebugFile);
+		cfg->DebugFile		 	 = strdup(DEFAULT_DEBUG_FILE);
 	} else {
 		cfg->DebugFile			 = strdup(cfg->DebugFile);
 		GSM_ExpandUserPath(&cfg->DebugFile);
 	}
 
 	/* Set file locking */
-	cfg->LockDevice  = INI_GetBool(cfg_info, section, "use_locking", DefaultLockDevice);
+	cfg->LockDevice  = INI_GetBool(cfg_info, section, "use_locking", DEFAULT_LOCK_DEVICE);
 
 	/* Set model */
 	Temp		 = INI_GetValue(cfg_info, section, "model", 		FALSE);
 	if (Temp == NULL || strcmp(Temp, "auto") == 0) {
-		strcpy(cfg->Model,DefaultModel);
+		strcpy(cfg->Model,DEFAULT_MODEL);
 	} else {
 		if (strlen(Temp) >= sizeof(cfg->Model))
 			Temp[sizeof(cfg->Model) - 1] = '\0';
@@ -1381,7 +1434,7 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 	Temp		 = INI_GetValue(cfg_info, section, "logformat", 	FALSE);
 
 	if (Temp == NULL) {
-		strcpy(cfg->DebugLevel,DefaultDebugLevel);
+		strcpy(cfg->DebugLevel,DEFAULT_DEBUG_LEVEL);
 	} else {
 		if (strlen(Temp) >= sizeof(cfg->DebugLevel))
 			Temp[sizeof(cfg->DebugLevel) - 1] = '\0';
@@ -1389,7 +1442,7 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 	}
 
 	/* Set startup info */
-	cfg->StartInfo = INI_GetBool(cfg_info, section, "startinfo", DefaultStartInfo);
+	cfg->StartInfo = INI_GetBool(cfg_info, section, "startinfo", DEFAULT_START_INFO);
 
 	/* Read localised strings for some phones */
 
@@ -1460,14 +1513,14 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 fail:
 	/* Special case, this config needs to be somehow valid */
 	if (num == 0) {
-		cfg->Device		 	 = strdup(DefaultPort);
-		cfg->Connection	 		 = strdup(DefaultConnection);
-		cfg->SyncTime		 	 = DefaultSynchronizeTime;
-		cfg->DebugFile		 	 = strdup(DefaultDebugFile);
-		cfg->LockDevice	 		 = DefaultLockDevice;
-		strcpy(cfg->Model,DefaultModel);
-		strcpy(cfg->DebugLevel,DefaultDebugLevel);
-		cfg->StartInfo	 		 = DefaultStartInfo;
+		cfg->Device		 	 = strdup(DEFAULT_DEVICE);
+		cfg->Connection	 		 = strdup(DEFAULT_CONNECTION);
+		cfg->SyncTime		 	 = DEFAULT_SYNCHRONIZE_TIME;
+		cfg->DebugFile		 	 = strdup(DEFAULT_DEBUG_FILE);
+		cfg->LockDevice	 		 = DEFAULT_LOCK_DEVICE;
+		strcpy(cfg->Model,DEFAULT_MODEL);
+		strcpy(cfg->DebugLevel,DEFAULT_DEBUG_LEVEL);
+		cfg->StartInfo	 		 = DEFAULT_START_INFO;
 		strcpy(cfg->TextReminder,"Reminder");
 		strcpy(cfg->TextMeeting,"Meeting");
 		strcpy(cfg->TextCall,"Call");
@@ -1480,7 +1533,7 @@ fail:
 	return error;
 }
 
-void GSM_DumpMessageLevel2_Text(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, const char *text)
+void GSM_DumpMessageText_Custom(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, const char *text)
 {
 	GSM_Debug_Info *curdi;
 
@@ -1490,24 +1543,24 @@ void GSM_DumpMessageLevel2_Text(GSM_StateMachine *s, unsigned const char *messag
 			curdi->dl == DL_TEXTALL ||
 			curdi->dl == DL_TEXTDATE ||
 			curdi->dl == DL_TEXTALLDATE) {
-		smprintf(s, "%s", text);
+		smprintf(s, "%s ", text);
 		smprintf(s, "type 0x%02X/length 0x%02X/%i",
 				type, messagesize, messagesize);
 		DumpMessage(curdi, message, messagesize);
 	}
 }
 
-void GSM_DumpMessageLevel2(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+void GSM_DumpMessageText(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
 {
-	GSM_DumpMessageLevel2_Text(s, message, messagesize, type, "SENDING frame");
+	GSM_DumpMessageText_Custom(s, message, messagesize, type, "SENDING frame");
 }
 
-void GSM_DumpMessageLevel2Recv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+void GSM_DumpMessageTextRecv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
 {
-	GSM_DumpMessageLevel2_Text(s, message, messagesize, type, "RECEIVED frame");
+	GSM_DumpMessageText_Custom(s, message, messagesize, type, "RECEIVED frame");
 }
 
-void GSM_DumpMessageLevel3_Custom(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, int direction)
+void GSM_DumpMessageBinary_Custom(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, int direction)
 {
 	int i=0;
 	GSM_Debug_Info *curdi;
@@ -1525,14 +1578,14 @@ void GSM_DumpMessageLevel3_Custom(GSM_StateMachine *s, unsigned const char *mess
 		}
 	}
 }
-void GSM_DumpMessageLevel3(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+void GSM_DumpMessageBinary(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
 {
-	GSM_DumpMessageLevel3_Custom(s, message, messagesize, type, 0x01);
+	GSM_DumpMessageBinary_Custom(s, message, messagesize, type, 0x01);
 }
 
-void GSM_DumpMessageLevel3Recv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+void GSM_DumpMessageBinaryRecv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
 {
-	GSM_DumpMessageLevel3_Custom(s, message, messagesize, type, 0x02);
+	GSM_DumpMessageBinary_Custom(s, message, messagesize, type, 0x02);
 }
 
 void GSM_OSErrorInfo(GSM_StateMachine *s, const char *description)
@@ -1623,6 +1676,9 @@ GSM_StateMachine *GSM_AllocStateMachine(void)
 {
 	GSM_StateMachine *ret;
 	ret = (GSM_StateMachine *)calloc(1, sizeof(GSM_StateMachine));
+	if (ret == NULL) {
+		return ret;
+	}
 	ret->CurrentConfig = &(ret->Config[0]);
 	ret->Abort = FALSE;
 	return ret;
