@@ -1,5 +1,5 @@
 /* Copyright (c) 2002-2004 by Marcin Wiacek and Joergen Thomsen */
-/* Copyright (c) 2009 - 2012 Michal Cihar <michal@cihar.com> */
+/* Copyright (c) 2009 - 2015 Michal Cihar <michal@cihar.com> */
 
 #include <string.h>
 #include <signal.h>
@@ -15,6 +15,7 @@
 #endif
 #include <stdarg.h>
 #include <stdlib.h>
+#include <fcntl.h>
 
 #include <gammu-smsd.h>
 
@@ -378,12 +379,15 @@ GSM_SMSDConfig *SMSD_NewConfig(const char *name)
 	Config->gammu_log_buffer_size = 0;
 	Config->logfilename = NULL;
 	Config->RunOnFailure = NULL;
+	Config->RunOnSent = NULL;
+	Config->RunOnReceive = NULL;
 	Config->smsdcfgfile = NULL;
 	Config->log_handle = NULL;
 	Config->log_type = SMSD_LOG_NONE;
 	Config->debug_level = 0;
 	Config->ServiceName = NULL;
 	Config->Service = NULL;
+	Config->IgnoredMessages = 0;
 
 #if defined(HAVE_MYSQL_MYSQL_H)
 	Config->conn.my = NULL;
@@ -684,14 +688,18 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 
 #ifdef HAVE_SHM
 	/* Calculate key for shared memory */
-	if (realpath(filename, fullpath) == NULL) {
+	if (filename == NULL) {
+		strcpy(fullpath, ":default:");
+	} else if (realpath(filename, fullpath) == NULL) {
 		strncpy(fullpath, filename, PATH_MAX);
 		fullpath[PATH_MAX] = 0;
 	}
 	Config->shm_key = ftok(fullpath, SMSD_SHM_KEY);
 #endif
 #ifdef WIN32
-	if (GetFullPathName(filename, sizeof(config_name), config_name, NULL) == 0) {
+	if (filename == NULL) {
+		strcpy(config_name, ":default:");
+	} else if (GetFullPathName(filename, sizeof(config_name), config_name, NULL) == 0) {
 		return FALSE;
 	}
 
@@ -779,7 +787,7 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 	Config->commtimeout = INI_GetInt(Config->smsdcfgfile, "smsd", "commtimeout", 30);
 	Config->deliveryreportdelay = INI_GetInt(Config->smsdcfgfile, "smsd", "deliveryreportdelay", 600);
 	Config->sendtimeout = INI_GetInt(Config->smsdcfgfile, "smsd", "sendtimeout", 30);
-	Config->receivefrequency = INI_GetInt(Config->smsdcfgfile, "smsd", "receivefrequency", 0);
+	Config->receivefrequency = INI_GetInt(Config->smsdcfgfile, "smsd", "receivefrequency", 15);
 	Config->statusfrequency = INI_GetInt(Config->smsdcfgfile, "smsd", "statusfrequency", 15);
 	Config->loopsleep = INI_GetInt(Config->smsdcfgfile, "smsd", "loopsleep", 1);
 	Config->checksecurity = INI_GetBool(Config->smsdcfgfile, "smsd", "checksecurity", TRUE);
@@ -820,6 +828,7 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 
 	Config->RunOnReceive = INI_GetValue(Config->smsdcfgfile, "smsd", "runonreceive", FALSE);
 	Config->RunOnFailure = INI_GetValue(Config->smsdcfgfile, "smsd", "runonfailure", FALSE);
+	Config->RunOnSent = INI_GetValue(Config->smsdcfgfile, "smsd", "runonsent", FALSE);
 
 	str = INI_GetValue(Config->smsdcfgfile, "smsd", "smsc", FALSE);
 	if (str) {
@@ -1116,10 +1125,19 @@ gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config, const char *locations)
 {
 	int pid;
+	int pipefd[2];
 	int i;
 	pid_t w;
 	int status;
 	char *cmdline;
+	ssize_t bytes;
+	char buffer[4097];
+	gboolean result = FALSE;
+
+	if (pipe(pipefd) == -1) {
+		SMSD_LogErrno(Config, "Failed to open pipe for child process!");
+		return FALSE;
+	}
 
 	pid = fork();
 
@@ -1130,12 +1148,22 @@ gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 
 	if (pid != 0) {
 		/* We are the parent, wait for child */
+
+		/* Close write end of pipe */
+		close(pipefd[1]);
+		fcntl(pipefd[0], F_SETFL, O_NONBLOCK);
+
 		i = 0;
 		do {
+			while ((bytes = read(pipefd[0], buffer, 4096)) > 0) {
+				buffer[bytes] = '\0';
+				SMSD_Log(DEBUG_INFO, Config, "Subprocess output: %s", buffer);
+			}
 			w = waitpid(pid, &status, WUNTRACED | WCONTINUED);
 			if (w == -1) {
 				SMSD_Log(DEBUG_INFO, Config, "Failed to wait for process");
-				return FALSE;
+				result = FALSE;
+				goto out;
 			}
 
 			if (WIFEXITED(status)) {
@@ -1144,10 +1172,12 @@ gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 				} else {
 					SMSD_Log(DEBUG_ERROR, Config, "Process failed with exit status %d", WEXITSTATUS(status));
 				}
-				return (WEXITSTATUS(status) == 0);
+				result = (WEXITSTATUS(status) == 0);
+				goto out;
 			} else if (WIFSIGNALED(status)) {
 				SMSD_Log(DEBUG_ERROR, Config, "Process killed by signal %d", WTERMSIG(status));
-				return FALSE;
+				result = FALSE;
+				goto out;
 			} else if (WIFSTOPPED(status)) {
 				SMSD_Log(DEBUG_INFO, Config, "Process stopped by signal %d", WSTOPSIG(status));
 			} else if (WIFCONTINUED(status)) {
@@ -1157,15 +1187,25 @@ gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 
 			if (i++ > 1200) {
 				SMSD_Log(DEBUG_INFO, Config, "Waited two minutes for child process, giving up");
-				return TRUE;
+				result = TRUE;
+				goto out;
 			}
 		} while (!WIFEXITED(status) && !WIFSIGNALED(status));
+out:
+		while ((bytes = read(pipefd[0], buffer, 4096)) > 0) {
+			buffer[bytes] = '\0';
+			SMSD_Log(DEBUG_INFO, Config, "Subprocess output: %s", buffer);
+		}
+		close(pipefd[0]);
 
-		return TRUE;
+		return result;
 	}
 
 	/* we are the child */
 
+	/* Close read end of pipe */
+	close(pipefd[0]);
+
 	/* Prepare environment */
 	if (sms != NULL) {
 		SMSD_RunOnReceiveEnvironment(sms, Config, locations);
@@ -1177,18 +1217,21 @@ gboolean SMSD_RunOn(const char *command, GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 
 	/* Close all file descriptors */
 	for (i = 0; i < 255; i++) {
-		close(i);
+		if (i != pipefd[1]) {
+			close(i);
+		}
 	}
 
+	/* Connect stdout and stderr to pipe */
+	dup2(pipefd[1], 1);
+	dup2(pipefd[1], 2);
+
 	/* Run the program */
-	status = system(cmdline);
+	execl("/bin/sh", "sh", "-c", cmdline, NULL);
 
-	/* Propagate error code */
-	if (WIFEXITED(status)) {
-		exit(WEXITSTATUS(status));
-	} else {
-		exit(127);
-	}
+	/* Happens only in case of error */
+	SMSD_LogErrno(Config, "Error executing new process");
+	exit(127);
 }
 #endif
 
@@ -1374,6 +1417,7 @@ gboolean SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config)
 	int i, j;
 
 	/* Read messages from phone */
+	Config->IgnoredMessages = 0;
 	start=TRUE;
 	sms.Number = 0;
 	sms.SMS[0].Location = 0;
@@ -1408,10 +1452,12 @@ gboolean SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config)
 					*(GetSMSData[GetSMSNumber]) = sms;
 					GetSMSNumber++;
 					GetSMSData[GetSMSNumber] = NULL;
+				} else {
+					Config->IgnoredMessages++;
 				}
 				break;
 			default:
-				SMSD_LogError(DEBUG_INFO, Config, "Error getting SMS", error);
+				SMSD_LogError(DEBUG_ERROR, Config, "Error getting SMS", error);
 				if (GetSMSData != NULL) {
 					for (i = 0; GetSMSData[i] != NULL; i++) {
 						free(GetSMSData[i]);
@@ -1503,7 +1549,7 @@ gboolean SMSD_CheckSMSStatus(GSM_SMSDConfig *Config)
 	/* First try SMS status */
 	error = GSM_GetSMSStatus(Config->gsm,&SMSStatus);
 	if (error == ERR_NONE) {
-		new_message = (SMSStatus.SIMUsed + SMSStatus.PhoneUsed > 0);
+		new_message = (SMSStatus.SIMUsed + SMSStatus.PhoneUsed - Config->IgnoredMessages > 0);
 	} else if (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED) {
 		/* Fallback to GetNext */
 		sms.Number = 0;
@@ -1585,24 +1631,17 @@ GSM_Error SMSD_SendSMS(GSM_SMSDConfig *Config)
 		return ERR_NONE;
 	}
 
-	if (Config->SMSID[0] != 0 && strcmp(Config->prevSMSID, Config->SMSID) == 0) {
-		SMSD_Log(DEBUG_NOTICE, Config, "Same message as previous one: %s", Config->SMSID);
-		Config->retries++;
-		if (Config->retries > Config->maxretries) {
-			Config->retries = 0;
-			strcpy(Config->prevSMSID, "");
-			SMSD_Log(DEBUG_INFO, Config, "Moved to errorbox: %s", Config->SMSID);
-			for (i=0;i<sms.Number;i++) {
-				Config->Status->Failed++;
-				Config->Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_ERROR, -1);
-			}
-			Config->Service->MoveSMS(&sms,Config, Config->SMSID, TRUE,FALSE);
-			return ERR_UNKNOWN;
+	if (Config->SMSID[0] != 0 && (Config->retries > Config->maxretries)) {
+		SMSD_Log(DEBUG_NOTICE, Config, "Moved to errorbox, reached MaxRetries: %s", Config->SMSID);
+		for (i=0;i<sms.Number;i++) {
+			Config->Status->Failed++;
+			Config->Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i + 1, SMSD_SEND_SENDING_ERROR, Config->TPMR);
 		}
+		Config->Service->MoveSMS(&sms,Config, Config->SMSID, TRUE,FALSE);
+		return ERR_UNKNOWN;
 	} else {
 		SMSD_Log(DEBUG_NOTICE, Config, "New message to send: %s", Config->SMSID);
-		Config->retries = 0;
-		strcpy(Config->prevSMSID, Config->SMSID);
+		Config->retries++;
 	}
 
 	for (i = 0; i < sms.Number; i++) {
@@ -1692,19 +1731,26 @@ GSM_Error SMSD_SendSMS(GSM_SMSDConfig *Config)
 		SMSD_LogError(DEBUG_ERROR, Config, "Error moving message", error);
 		Config->Service->MoveSMS(&sms,Config, Config->SMSID, TRUE, FALSE);
 	}
+
+	if (Config->RunOnSent != NULL && error == ERR_NONE) {
+		SMSD_RunOn(Config->RunOnSent, &sms, Config, Config->SMSID);
+	}
+
 	return ERR_NONE;
 failure_unsent:
 	if (Config->RunOnFailure != NULL) {
 		SMSD_RunOn(Config->RunOnFailure, NULL, Config, Config->SMSID);
 	}
 	Config->Status->Failed++;
-	Config->Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i + 1, SMSD_SEND_SENDING_ERROR, Config->TPMR);
-	Config->Service->MoveSMS(&sms,Config, Config->SMSID, TRUE, FALSE);
+
+	Config->Service->UpdateRetries(Config, Config->SMSID);
+
+       sleep(60);
 	return ERR_UNKNOWN;
 failure_sent:
-	if (Config->Service->MoveSMS(&sms,Config, Config->SMSID, FALSE, TRUE) != ERR_NONE) {
-		Config->Service->MoveSMS(&sms,Config, Config->SMSID, TRUE, FALSE);
-	}
+
+	Config->Service->UpdateRetries(Config, Config->SMSID);
+
 	return ERR_UNKNOWN;
 }
 
@@ -1718,7 +1764,11 @@ GSM_Error SMSD_InitSharedMemory(GSM_SMSDConfig *Config, gboolean writable)
 	Config->shm_handle = shmget(Config->shm_key, sizeof(GSM_SMSDStatus), writable ? (IPC_CREAT | S_IRWXU | S_IRGRP | S_IROTH) : 0);
 	if (Config->shm_handle == -1) {
 		SMSD_Terminate(Config, "Failed to allocate shared memory segment!", ERR_NONE, TRUE, -1);
-		return ERR_UNKNOWN;
+		if (writable) {
+			return ERR_UNKNOWN;
+		} else {
+			return ERR_NOTRUNNING;
+		}
 	}
 	Config->Status = shmat(Config->shm_handle, NULL, 0);
 	if (Config->Status == (void *) -1) {
