@@ -3,7 +3,7 @@
  *
  * Part of Gammu project
  *
- * Copyright (c) 2009 - 2011 Michal Cihar <michal@cihar.com>
+ * Copyright (c) 2009 - 2015 Michal Cihar <michal@cihar.com>
  *
  * Licensed under GNU GPL version 2 or later
  */
@@ -58,6 +58,10 @@ long long SMSDDBI_GetNumber(GSM_SMSDConfig * Config, SQL_result *res, unsigned i
 			}
 			SMSD_Log(DEBUG_ERROR, Config, "Wrong decimal field subtype from DBI: %d", type);
 			return -1;
+#ifdef DBI_TYPE_XDECIMAL
+		case DBI_TYPE_XDECIMAL:
+			return dbi_result_get_as_longlong_idx(res->dbi, field);
+#endif
 		default:
 			SMSD_Log(DEBUG_ERROR, Config, "Wrong field type for number (not INTEGER nor DECIMAL) from DBI: %d", type);
 			return -1;
@@ -75,6 +79,9 @@ time_t SMSDDBI_GetDate(GSM_SMSDConfig * Config, SQL_result *res, unsigned int fi
 	switch (type) {
 		case DBI_TYPE_INTEGER:
 		case DBI_TYPE_DECIMAL:
+#ifdef DBI_TYPE_XDECIMAL
+		case DBI_TYPE_XDECIMAL:
+#endif
 			return SMSDDBI_GetNumber(Config, res, field);
 		case DBI_TYPE_STRING:
 			date = dbi_result_get_string_idx(res->dbi, field);
@@ -100,6 +107,9 @@ gboolean SMSDDBI_GetBool(GSM_SMSDConfig * Config, SQL_result *res, unsigned int
 	switch (type) {
 		case DBI_TYPE_INTEGER:
 		case DBI_TYPE_DECIMAL:
+#ifdef DBI_TYPE_XDECIMAL
+		case DBI_TYPE_XDECIMAL:
+#endif
 			num = SMSDDBI_GetNumber(Config, res, field);
 			if (num == -1) {
 				return -1;
