@@ -3,7 +3,7 @@
  *
  * Part of Gammu project
  *
- * Copyright (C) 2011 Michal Čihař
+ * Copyright (C) 2011 - 2015 Michal Čihař
  *
  * Licensed under GNU GPL version 2 or later
  */
@@ -13,7 +13,7 @@
 #ifdef WIN32
 #include <windows.h>
 #ifndef __GNUC__
-#pragma comment(lib, "libodbc32.lib")
+#pragma comment(lib, "odbc32.lib")
 #endif
 #endif
 
@@ -21,6 +21,7 @@
 #include <sql.h>
 #include <sqlext.h>
 
+#include "../../helper/string.h"
 #include "../core.h"
 #include "sql.h"
 #include "sql-core.h"
@@ -47,7 +48,7 @@ static void SMSDODBC_LogError(GSM_SMSDConfig * Config, SQLRETURN origret, SQLSMA
 long long SMSDODBC_GetNumber(GSM_SMSDConfig * Config, SQL_result *res, unsigned int field)
 {
 	SQLRETURN ret;
-	SQLINTEGER value;
+	SQLINTEGER value = -1;
 
 	ret = SQLGetData(res->odbc, field + 1, SQL_C_SLONG, &value, 0, NULL);
 	if (!SQL_SUCCEEDED(ret)) {
@@ -101,7 +102,7 @@ const char *SMSDODBC_GetString(GSM_SMSDConfig * Config, SQL_result *res, unsigne
 	/*
 	 * This hack seems to be needed to avoid type breakage on Win64, don't ask me why.
 	 *
-	 * Might be actually bug in MinGW compiler, but when using SQLLEN type bellow
+	 * Might be actually bug in MinGW compiler, but when using SQLLEN type below
 	 * anything fails (it does not match to SQL_NULL_DATA and realloc always fails).
 	 */
 	size = sqllen;
@@ -273,7 +274,12 @@ char * SMSDODBC_QuoteString(GSM_SMSDConfig * Config, const char *string)
 		driver_name = Config->driver;
 	}
 
-	if (strcasecmp(driver_name, "access") == 0) {
+	if (strcasecmp(driver_name, "mysql") == 0 ||
+			strcasecmp(driver_name, "native_mysql") == 0 ||
+			strcasecmp(driver_name, "pgsql") == 0 ||
+			strcasecmp(driver_name, "native_pgsql") == 0 ||
+			strncasecmp(driver_name, "sqlite", 6) == 0 ||
+			strcasecmp(Config->driver, "access") == 0) {
 		quote = '\'';
 	}
 
