@@ -3,7 +3,7 @@
  *
  * Part of Gammu project
  *
- * Copyright (C) 2009-2011 Michal Čihař
+ * Copyright (C) 2009 - 2015 Michal Čihař
  *           (c) 2010 Miloslav Semler
  *
  * Licensed under GNU GPL version 2 or later
@@ -46,7 +46,7 @@ const char *SMSDSQL_SQLName(GSM_SMSDConfig * Config)
 const char now_plus_odbc[] = "{fn CURRENT_TIMESTAMP()} + INTERVAL %d SECOND";
 const char now_plus_mysql[] = "(NOW() + INTERVAL %d SECOND) + 0";
 const char now_plus_pgsql[] = "now() + interval '%d seconds'";
-const char now_plus_sqlite[] = "datetime('now', '+%d seconds')";
+const char now_plus_sqlite[] = "datetime('now', '+%d seconds', 'localtime')";
 const char now_plus_freetds[] = "DATEADD('second', %d, CURRENT_TIMESTAMP)";
 const char now_plus_access[] = "now()+#00:00:%d#";
 const char now_plus_fallback[] = "NOW() + INTERVAL %d SECOND";
@@ -149,7 +149,7 @@ static const char *SMSDSQL_LimitClause(GSM_SMSDConfig * Config, const char *coun
 const char now_odbc[] = "{fn CURRENT_TIMESTAMP()}";
 const char now_mysql[] = "NOW()";
 const char now_pgsql[] = "now()";
-const char now_sqlite[] = "datetime('now')";
+const char now_sqlite[] = "datetime('now', 'localtime')";
 const char now_freetds[] = "CURRENT_TIMESTAMP";
 const char now_access[] = "now()";
 const char now_fallback[] = "NOW()";
@@ -157,7 +157,7 @@ const char now_fallback[] = "NOW()";
 const char currtime_odbc[] = "{fn CURTIME()}";
 const char currtime_mysql[] = "CURTIME()";
 const char currtime_pgsql[] = "localtime";
-const char currtime_sqlite[] = "time('now')";
+const char currtime_sqlite[] = "time('now', 'localtime')";
 const char currtime_freetds[] = "CURRENT_TIME";
 const char currtime_fallback[] = "CURTIME()";
 
@@ -203,10 +203,30 @@ static const char *SMSDSQL_Now(GSM_SMSDConfig * Config)
 		return now_fallback;
 	}
 }
+
+static SQL_Error SMSDSQL_Reconnect(GSM_SMSDConfig * Config)
+{
+	SQL_Error error = SQL_TIMEOUT;
+	int attempts;
+	struct GSM_SMSDdbobj *db = Config->db;
+
+	SMSD_Log(DEBUG_INFO, Config, "Reconnecting to the database!");
+	for (attempts = 1; attempts <= Config->backend_retries; attempts++) {
+		SMSD_Log(DEBUG_INFO, Config, "Reconnecting after %d seconds...", attempts * attempts);
+		sleep(attempts * attempts);
+		db->Free(Config);
+		error = db->Connect(Config);
+		if (error == SQL_OK) {
+			return error;
+		}
+	}
+	return error;
+}
+
 static SQL_Error SMSDSQL_Query(GSM_SMSDConfig * Config, const char *query, SQL_result * res)
 {
 	SQL_Error error = SQL_TIMEOUT;
-	int attempts = 1;
+	int attempts;
 	struct GSM_SMSDdbobj *db = Config->db;
 
 	for (attempts = 1; attempts <= Config->backend_retries; attempts++) {
@@ -223,13 +243,9 @@ static SQL_Error SMSDSQL_Query(GSM_SMSDConfig * Config, const char *query, SQL_r
 
 		SMSD_Log(DEBUG_INFO, Config, "SQL failed (timeout): %s", query);
 		/* We will try to reconnect */
-		SMSD_Log(DEBUG_INFO, Config, "reconnecting to database!");
-		while (error != SQL_OK && attempts < Config->backend_retries) {
-			SMSD_Log(DEBUG_INFO, Config, "Reconnecting after %d seconds...", attempts * attempts);
-			sleep(attempts * attempts);
-			db->Free(Config);
-			error = db->Connect(Config);
-			attempts++;
+		error = SMSDSQL_Reconnect(Config);
+		if (error != SQL_OK) {
+			break;
 		}
 	}
 	return error;
@@ -280,7 +296,11 @@ static SQL_Error SMSDSQL_NamedQuery(GSM_SMSDConfig * Config, const char *sql_que
 	NetCode = empty;
 	NetName = empty;
 
-	if ( GSM_GetNetworkInfo(Config->gsm, &NetInfo) == ERR_NONE) {
+	/* Query network status only if we need it */
+	if (
+		(strstr(sql_query, "%O") != NULL || strstr(sql_query, "%M") != NULL)
+		&& GSM_GetNetworkInfo(Config->gsm, &NetInfo) == ERR_NONE
+	) {
 		NetCode = NetInfo.NetworkCode;
 		if (NetInfo.NetworkName[0] != 0x00 || NetInfo.NetworkName[1] != 0x00) {
 			NetName = DecodeUnicodeConsole(NetInfo.NetworkName);
@@ -611,6 +631,7 @@ static GSM_Error SMSDSQL_SaveInboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConfig
 
 				if (strcmp(smsc, smsc_message) != 0) {
 					if (Config->skipsmscnumber[0] == 0 || strcmp(Config->skipsmscnumber, smsc)) {
+						SMSD_Log(DEBUG_ERROR, Config, "Failed to match SMSC, you might want to use SkipSMSCNumber (sent: %s, received: %s)", smsc_message, smsc);
 						continue;
 					}
 				}
@@ -664,6 +685,8 @@ static GSM_Error SMSDSQL_SaveInboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConfig
 					return ERR_UNKNOWN;
 				}
 				db->FreeResult(Config, &res2);
+			} else {
+				SMSD_Log(DEBUG_ERROR, Config, "Failed to find SMS for TPMR=%i, Number=%s", sms->SMS[i].MessageReference, sms->SMS[i].Number);
 			}
 			db->FreeResult(Config, &res);
 			continue;
@@ -731,6 +754,32 @@ static GSM_Error SMSDSQL_RefreshSendStatus(GSM_SMSDConfig * Config, char *ID)
 	return ERR_NONE;
 }
 
+
+
+static GSM_Error SMSDSQL_UpdateRetries(GSM_SMSDConfig * Config, char *ID)
+{
+	SQL_result res;
+	struct GSM_SMSDdbobj *db = Config->db;
+	SQL_Var vars[3] = {
+		{SQL_TYPE_STRING, {ID}},
+		{SQL_TYPE_INT, {NULL}},
+		{SQL_TYPE_NONE, {NULL}}};
+	vars[1].v.i = Config->retries;
+
+	if (SMSDSQL_NamedQuery(Config, Config->SMSDSQL_queries[SQL_QUERY_UPDATE_RETRIES], NULL, vars, &res) != SQL_OK) {
+		SMSD_Log(DEBUG_INFO, Config, "Error writing to database (%s)", __FUNCTION__);
+		return ERR_UNKNOWN;
+	}
+
+	if (db->AffectedRows(Config, &res) == 0) {
+		db->FreeResult(Config, &res);
+		return ERR_UNKNOWN;
+	}
+
+	db->FreeResult(Config, &res);
+	return ERR_NONE;
+}
+
 /* Find one multi SMS to sending and return it (or return ERR_EMPTY)
  * There is also set ID for SMS
  */
@@ -888,6 +937,7 @@ static GSM_Error SMSDSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConfig
 			Config->relativevalidity = db->GetNumber(Config, &res, 8);
 
 			Config->currdeliveryreport = db->GetBool(Config, &res, 9);
+			Config->retries = db->GetNumber(Config, &res, 11);
 
 			/* Is this a multipart message? */
 			if (!db->GetBool(Config, &res, 7)) {
@@ -1284,6 +1334,14 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 		return ERR_UNKNOWN;
 	}
 
+	if (SMSDSQL_option(Config, SQL_QUERY_UPDATE_RETRIES, "update_retries",
+		"UPDATE outbox SET ",
+			ESCAPE_FIELD("SendingTimeOut"), " = ", SMSDSQL_NowPlus(Config, 600),
+			", ",ESCAPE_FIELD("Retries"), " = %2"
+			" WHERE ", ESCAPE_FIELD("ID"), " = %1", NULL) != ERR_NONE) {
+		return ERR_UNKNOWN;
+	}
+
 	if (SMSDSQL_option(Config, SQL_QUERY_FIND_OUTBOX_SMS_ID, "find_outbox_sms_id",
 		"SELECT ", SMSDSQL_TopClause(Config, "%1"),
 			ESCAPE_FIELD("ID"),
@@ -1313,6 +1371,7 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 			", ", ESCAPE_FIELD("RelativeValidity"),
 			", ", ESCAPE_FIELD("DeliveryReport"),
 			", ", ESCAPE_FIELD("CreatorID"),
+			", ", ESCAPE_FIELD("Retries"),
 			" FROM outbox WHERE ",
 			ESCAPE_FIELD("ID"), "=%1", NULL) != ERR_NONE) {
 		return ERR_UNKNOWN;
@@ -1460,6 +1519,7 @@ GSM_SMSDService SMSDSQL = {
 	SMSDSQL_CreateOutboxSMS,
 	SMSDSQL_AddSentSMSInfo,
 	SMSDSQL_RefreshSendStatus,
+	SMSDSQL_UpdateRetries,
 	SMSDSQL_RefreshPhoneStatus,
 	SMSDSQL_ReadConfiguration
 };
